#ifndef _GAMMA_H
#define _GAMMA_H

#include "../include/color.h"

static const int G_Domain = 256;        // Input domain
static const int G_Range = 256;         // Output range

class Gamma     // Gamma correction
{
  private:
	 // Gamma correction lookup table
  public:
	 static BYTE GammaTable[256];

	 double g_value;     // Gamma value

	 void InitTable()
	 {
		int i;    // Loop index

		// Calculate gamma correction lookup table entries
		for (i = 0; i < G_Domain; i++)
		  GammaTable[i] = (BYTE) ((double) G_Range *
				pow((double) i / (double) G_Domain, 1.0 /
				g_value));
	 }

	 Gamma( double g = 2.2 )
	 {
		g_value = g;
		InitTable();
	 }

	 double GetGamma() { return g_value; }

	 void Correct( ColorRGB &c )
	 {      
		c.SetRed(GammaTable[c.GetRed()]);
		c.SetGreen(GammaTable[c.GetGreen()]);
		c.SetBlue(GammaTable[c.GetBlue()]);
	 }

	 void SetGamma( double g )
	 {
		g_value = g;
		InitTable();
	 }
};

#endif
