#ifndef _SYN_CAM_H
#define _SYN_CAM_H

#include "../source/p_clip4.c"
#include "../source/gamma.c"
#include "../source/c_jitter.c"
#include "../source/p_render.c"
//#include "win_meta.h"
//#include "win_bmap.h"
#include "../source/view_sys.c"

// Supersampling resolution identifiers
enum SC_SuperRes
{ SC_None, SC_Bartlett_3, SC_Bartlett_5, SC_Bartlett_7 };

// Color type definitions
static const int SC_RGB = 0;       // RGB color
static const int SC_MONO = 1;      // Grayscale
static const int SC_PSEUDO = 2;    // Pseudocolor

// Extents fill factor
static const double SC_FILL_FACTOR = 0.9;

//#ifdef _NOT_WINAPP
/*struct POINT    // Raster display point
{
  int x;        // X-axis co-ordinate
  int y;        // Y-axis co-ordinate
};*/
//#endif

struct Bartlett         // Bartlett filter element
{
  double x_offset;      // x-axis subpixel offset
  double y_offset;      // y-axis subpixel offset
  int alpha;            // Subpixel blend coefficient
  int beta;             // Pixel blend coefficient
  int weight;           // Subpixel weight
};

struct BartParam        // Bartlett filter parameters
{
  double offset;        // Subpixel offset
  int width;            // Filter array width
  char *psize;          // Filter size pointer
  char *pname;          // Filter name pointer
};

class SynCamera : public ViewSys        // Synthetic camera
{
  private:
    int color_type;         // Display color type
    int height;             // Window height
    int width;              // Window width
    int num_sub;            // Number of subpixels
    int pass;               // Number of passes
    int ss_res;             // Supersampling resolution
    BOOL fp_flag;           // First pass flag
    BOOL gamma_flag;        // Gamma correction flag
    BOOL jitter_flag;       // Color reduction flag
    Bartlett *pbart;        // Bartlett filter pointer
    ColorJitter jitter;     // Color reduction filter
    ColorRGB **pabuf;       // Antialiasing buffer pointer
    Gamma gamma;            // Gamma correction object
    PolyClip4 clipper;      // Polygon clipper
    PolyRender renderer;    // Polygon renderer

    // Bartlett filter parameters array
    static BartParam BartlettParam[];

    BOOL InstBartlettFilter();
    BOOL OpenBuffer(int x ,int y/* WinBitmap * */ );
    void CloseBuffer();
    void RenderPolygons( Environ *, int, double, double );

  public:
    SynCamera( int w = 640, int h = 480 ) : ViewSys()
    {
      width = w; height = h;
      aspect = (double) w / (double) h;
      fp_flag = TRUE;
      color_type = SC_RGB;
      pass = 0;
      ss_res = SC_None;
      gamma_flag = TRUE;
      jitter_flag = FALSE;
    }

    BOOL AntiAliasFlag()
    { return ss_res != SC_None ? TRUE: FALSE; }
  //  BOOL ContrastFlag() { return renderer.ContrastFlag(); }
    BOOL GammaFlag() { return gamma_flag; }
    BOOL GetStatus() { return jitter.GetStatus(); }
 //   BOOL IntensityFlag()
 //   { return renderer.IntensityFlag(); }
    BOOL JitterFlag() { return jitter_flag; }
 //   BOOL Preview( Environ *, WinMetaFile * );
 //   BOOL Shade( Environ *, WinBitmap * );
    BOOL Shoot( Environ *,int x, int y/* WinBitmap * */, BOOL * );
    char *GetCurrFilterName();
    char *GetCurrFilterSize();
    char *GetFilterSize( int );
  //  double GetContrast() { return renderer.GetContrast(); }
    double GetGamma() { return gamma.GetGamma(); }
 //   double GetIntensity()
 //   { return renderer.GetIntensity(); }
    int GetColorType() { return color_type; }
    int GetHeight() { return height; }
    int GetWidth() { return width; }
    int GetNoiseLevel() { return jitter.GetNoiseLevel(); }
    int GetMaxPass();
    int GetNumFilters();
    int GetNumPass() { return pass; }
    void Abort();
    void DisableContrast() { renderer.DisableContrast(); }
    void DisableGamma() { gamma_flag = FALSE; }
    void DisableIntensity() { renderer.DisableIntensity(); }
    void DisableJitter() { jitter_flag = FALSE; }
    void EnableContrast() { renderer.EnableContrast(); }
    void EnableGamma() { gamma_flag = TRUE; }
    void EnableIntensity() { renderer.EnableIntensity(); }
    void EnableJitter() { jitter_flag = TRUE; }
    void Reset();
    void SetFilterType( int t ) { ss_res = t; }
    void SetColorType( int t ) { color_type = t; }
    void SetContrast( double c ) { renderer.SetContrast(c); }
    void SetGamma( double g ) { gamma.SetGamma(g); }
    void SetHeight( int h ) { height = h; }
    void SetIntensity( double i )  { renderer.SetIntensity(i); }
    void SetNoiseLevel( int n ) { jitter.SetNoiseLevel(n); }
    void SetWidth( int w ) { width = w; }
    void UpdateViewSystem();
};

#endif


