#include "../include/ct_delta.h"

// Static delta form factor row pointer array
float *CubicDelta::delta_array[CT_DeltaDim];

// Static delta form factor array
float CubicDelta::ff_array[CT_FormDim];

CubicDelta::CubicDelta()        // Class constructor
{
  int i = 0;                // Form factor array index
  int left, right;          // Index boundaries
  int row, col;             // Current indices
  double delta;             // Cell width
  double diag_delta;        // Diagonal cell width
  double area;              // Cell area
  double diag_area;         // Diagonal cell area
  double y, z;              // Cell center
  double diag_y, diag_z;    // Diagonal cell center
  double r2;                // Cell distance squared
  double sqrt_3;			// Square root of three

  // Calculate square root of three
  sqrt_3 = sqrt(3.0);

  // Initialize index boundaries
  left = 0;
  right = FF_ArrayRes - 1;

  // Initialize cell values
  delta = (CT_MaxCoord - CT_MinCoord) / FF_ArrayRes;
  diag_delta = delta / 2.0;
  area  = delta * delta;
  diag_area  = area / 2.0;
  y = z = CT_MaxCoord - diag_delta;

  // Calculate delta form factors
  for (row = 0; row < CT_DeltaDim; row++)
  {
	 // Save delta form factor array row pointer
	 delta_array[row] = &(ff_array[i]);

	 for (col = left; col < right; col++)
	 {
		// Calculate square of cell distance
		r2 = y * y + z * z + 1;

		// Calculate cell delta form factor
		ff_array[i++] = (float) (area * (y + z + 1) / (PI * r2
			 * r2 * sqrt_3));

		y -= delta;
	 }

	 // Calculate square of diagonal cell distance
	 diag_y = y + diag_delta;
	 diag_z = z + diag_delta;
	 r2 = diag_y * diag_y + diag_z * diag_z + 1;

	 // Calculate diagonal cell delta form factor
	 ff_array[i++] = (float) (diag_area * (diag_y + diag_z +
		  1) / (PI * r2 * r2 * sqrt_3));

	 left++;
	 right--;
	 y = z -= delta;
  }
}


